
-- # import Property
-- # import Maybe
-- # import Monad
-- # import Functor
-- # import Monad.Proofs

propMaybeNothingJust : (A : Set) -> (m : Maybe A) ->
                        (maybe nothing just m) == m
propMaybeNothingJust A nothing  = reflexivity nothing
propMaybeNothingJust A (just x) = reflexivity (just x)

-- I would like this to be local to monadMaybe.
propMonadMaybe3 : (A, B, C : Set) ->
                   (m : Maybe A) -> (g : A -> Maybe B) -> (h : B -> Maybe C) ->
                      (maybe nothing h (maybe nothing g m))
                      ==
                      (maybe nothing
                        (\x -> maybe nothing h (g x))
                        m
                      )
propMonadMaybe3 A B C nothing  g h = reflexivity nothing
propMonadMaybe3 A B C (just x) g h = reflexivity (maybe nothing h (g x))

monadMaybe : Monad Maybe
monadMaybe = struct
    ops    = struct {return=return;bind=bind}
    proofs = laws
  where
    return : (A : Set) -> A -> Maybe A
    return A = just

    bind : (A, B : Set) -> Maybe A -> (A -> Maybe B) -> Maybe B
    bind A B m k = maybe nothing k m

    laws : MonadLaws Maybe return bind
    laws = struct 
      law1 = \A m -> propMaybeNothingJust A m
      law2 = \A B x f -> reflexivity (f x)
      law3 = propMonadMaybe3

functorMaybe : Functor Maybe
functorMaybe = functorMonad Maybe monadMaybe

-- Equ Maybe --------------------------------------------------------------

eqMaybe : (A : Set) -> Eq A -> Maybe A -> Maybe A -> Bool
eqMaybe A equA nothing nothing    = true
eqMaybe A equA (just x) nothing   = false
eqMaybe A equA nothing (just y)   = false
eqMaybe A equA (just x) (just y)  = equA.eq x y

eqMaybeRel : (A : Set) -> Eq A -> Relation (Maybe A)
eqMaybeRel A equA = decidableRelation (eqMaybe A equA)

reflEqMaybe  : (A : Set) -> (equA : Eq A) -> Reflexive  (eqMaybeRel A equA)

symEqMaybe   : (A : Set) -> (equA : Eq A) -> Symmetric  (eqMaybeRel A equA)

transEqMaybe : (A : Set) -> (equA : Eq A) -> Transitive (eqMaybeRel A equA)

equMaybe : (A : Set) -> Eq A -> Eq (Maybe A)
equMaybe A equA = struct
  eq     = eqMaybe A equA
  proofs = struct
    refl  = reflEqMaybe A equA
    sym   = symEqMaybe A equA
    trans = transEqMaybe A equA

-- Ord Maybe --------------------------------------------------------------

leqMaybe : (A : Set) -> Ord A -> Maybe A -> Maybe A -> Bool
leqMaybe A ordA nothing m         = true
leqMaybe A ordA (just x) nothing  = false
leqMaybe A ordA (just x) (just y) = ordA.leq x y

leqMaybeRel : (A : Set) -> Ord A -> Relation (Maybe A)
leqMaybeRel A ordA = decidableRelation (leqMaybe A ordA)

reflLeqMaybe : (A : Set) -> (ordA : Ord A) ->
                Reflexive' (eqMaybeRel  A ordA.equ)
                           (leqMaybeRel A ordA)
antisymLeqMaybe : (A : Set) -> (ordA : Ord A) ->
                   AntiSymmetric' (eqMaybeRel A ordA.equ)
                                  (leqMaybeRel A ordA)

transLeqMaybe : (A : Set) -> (ordA : Ord A) ->
                 Transitive (leqMaybeRel A ordA)

totalLeqMaybe : (A : Set) -> (ordA : Ord A) ->
                 Total (leqMaybeRel A ordA)

ordMaybe : (A : Set) -> Ord A -> Ord (Maybe A)
ordMaybe A ordA = struct
  leq = leqMaybe A ordA
  equ = equMaybe A ordA.equ
  proofs = struct
    refl     = reflLeqMaybe A ordA
    antisym  = antisymLeqMaybe A ordA
    trans    = transLeqMaybe A ordA
    total    = totalLeqMaybe A ordA
