-- Properties universe polymorphism version
-- Authors:: Ulf Norell
-- Created:: 2004-11-16

-- Prop provides codes for quantifier free propositions.

-- proofs

-- predicates

-- p : Pred <=> p = (xs:As) -> Holds (d xs) : Type
-- Verify (es) : t  if t : Pred and ei : pi : Pred for all i

-- atoms

Eq : (A : Type) -> A -> A -> Prop

reflexivity : (A:Set) -> (x:A) -> Eq A x x
reflexivity A = fol-plugin()

transitivity : (A:Set) -> (x,y,z : A) -> Eq A x y -> Eq A y z -> Eq A x z
transitivity A = fol-plugin()

symmetry : (A:Set) -> (x,y:A) -> Eq A x y -> Eq A y x
symmetry A = fol-plugin()

substitutivity : (A:Set) -> (P : A -> Prop) -> (x, y : A) ->
                  Eq A x y -> P x -> P y
substitutivity A P = fol-plugin()

congruence : (A, B : Set) -> (f : A -> B) -> (x, y : A) ->
                  Eq A x y -> Eq B (f x) (f y)
congruence A B f x y xy =
    substitutivity A  (\z -> Eq B (f x) (f z))
                      x y xy
                      (reflexivity B (f x))

-- connectives

{-
At the moment it's ok to define p ==> q = p -> q since the typing rule for
Pi states

S1 sort
S2 sort
A : S1
B : S2 [x : A]
S = lub S1 S2
-------------------
(x:A) -> B[x] : S

We might want to restrict Pi to only accept sorts of the form #i,
in which case this definition would no longer be valid.

-}

data True : Prop where
  tt : True

data False : Prop where

(==>) : Prop -> Prop -> Prop
p ==> q = p -> q

(<==) : Prop -> Prop -> Prop
p <== q = q ==> p

Not : Prop -> Prop
Not P = P ==> False

(/\) : Prop -> Prop -> Prop
P /\ Q = { P ; Q }

--data (/\) (P, Q : Prop) : Prop where
--  andIntro : P -> Q -> P /\ Q

{-
data (\/) (P, Q : Prop) : Prop where
  OrIntro1 : P -> P \/ Q
  OrIntro2 : Q -> P \/ Q
-}

-- Classical or
(\/) : Prop -> Prop -> Prop
P \/ Q = Not (Not P /\ Not Q)

(<=>) : Prop -> Prop -> Prop
P <=> Q = (P ==> Q) /\ (Q ==> P)

ForAll : (A:Set) -> (A -> Prop) -> Prop
ForAll A B = (x:A) -> B x

Exists : (A:Set) -> (A -> Prop) -> Prop
Exists A B = Not (ForAll A (\x -> Not (B x)))

infixl 4 /\
infixl 3 \/
infixr 2 ==>, <==
infix 1 <=>

-- Proof rules

--raa   : (p:Prop) -> False -> p

--andI : (P:Prop) -> (Q:Prop) -> P ==> Q ==> P /\ Q
--andI P Q = fol-plugin ()
{-
andElim1 : (P:Prop) -> (Q:Prop) -> P /\ Q ==> P
andElim1 P Q pq = pq.1

andElim2 : (P:Prop) -> (Q:Prop) -> P /\ Q ==> Q
andElim2 P Q pq = pq.2

falseElim : (P:Prop) -> False ==> P
falseElim P = fol-plugin()
-}
