
module Plugins.FOL.Options where

import Control.Monad.Error
import System.Console.GetOpt

import Interface.Options (Flag)
import Utils.Monads

data FOLOptions =
    FOLOptions	{ optTimeout	    :: Int
		, optProver	    :: Maybe String
		, optFinder	    :: Maybe String
		, optKeepFiles	    :: Bool
		, optCounterModels  :: Bool
		, optBinaryApp	    :: Bool
		, optModelSize	    :: Int
		}

folFinderFlag p o   = return $ o { optFinder	    = Just p }
folProverFlag p o   = return $ o { optProver	    = Just p }
keepFilesFlag o	    = return $ o { optKeepFiles	    = True }
counterModelsFlag o = return $ o { optCounterModels = True }
binaryAppFlag o	    = return $ o { optBinaryApp	    = True }
modelSizeFlag s o   =
    do  n <- readM s
	return o { optModelSize = n }
    `catchError`
	\_ -> fail "option '--fol-model-size requires an int argument\n"
timeoutFlag s o	    =
    do  n <- readM s
	return o { optTimeout = n }
    `catchError`
	\_ -> fail "option '--fol-timeout' requires an int argument\n"

defaultOptions :: FOLOptions
defaultOptions =
    FOLOptions	{ optTimeout	    = 3000
		, optProver	    = Nothing
		, optKeepFiles	    = False
		, optFinder	    = Nothing
		, optCounterModels  = False
		, optBinaryApp	    = False
		, optModelSize	    = 8
		}

options :: [OptDescr (Flag FOLOptions)]
options =
    [ Option [] ["timeout"]	(ReqArg timeoutFlag "N")
		"set timeout for first order prover to N milliseconds"
    , Option []	["prover"]	(ReqArg folProverFlag "PROVER")
		"use PROVER as the first order prover"
    , Option [] ["model-finder"] (ReqArg folFinderFlag "FINDER")
		"use the model finder FINDER to find counter models"
    , Option [] ["counter-models"] (NoArg counterModelsFlag)
		"search for counter models if proving fails"
    , Option [] ["model-size"] (ReqArg modelSizeFlag "N")
		"set maximum counter model size to N (default=8)"
    , Option []	["keep-files"]	(NoArg keepFilesFlag)
		"do not remove intermediate files"
    , Option [] ["binary-app"]   (NoArg binaryAppFlag)
		"use binary applicate instead of function symbols"
    ]

