{-| This should probably be in a different directory.
-}

module TypeChecker.Haskell where

import Syntax.Internal
import Syntax.Values

import TypeChecker.Values
import TypeChecker.UnfoldConstants
import TypeChecker.Monad
import TypeChecker.Monad.Errors
import TypeChecker.Errors
import TypeChecker.Abstractions

import Utils.Error
import Utils.Monads

-- Class definitions ------------------------------------------------------

-- | The value should be fully unfolded and contain no meta variables.
class Extractable a where
    toHaskell :: Value -> a

class Injectable a where
    fromHaskell	:: a -> Value

-- Value ------------------------------------------------------------------

instance Extractable Value where
    toHaskell = id

instance Injectable Value where
    fromHaskell = id

-- a -> b -----------------------------------------------------------------

instance (Injectable a, Extractable b) => Extractable (a -> b) where
    toHaskell v = toHaskell . vApply v . fromHaskell

{- We can't do this without higher order values! Or primitive functions.
instance (Extractable a, Injectable b) => Injectable (a -> b) where
    fromHaskell t f = VLam (mkCl "x" v app) t
	where
	    VPi v cl _ = unfoldConstants t
	    app = fromHaskell w $ f (toHaskell v)
		where
		    w = cl `clApp` v
-}

-- Int --------------------------------------------------------------------

{-
v_zero, v_succ, v_Nat, v_Set :: Value
v_Set	= vSort (VSUniv UZero)
v_Nat	= vCon (dummyName "Nat") 0 0
v_zero	= vCon (dummyName "zero") 0 0
v_succ	= vCon (dummyName "succ") 0	1

instance Extractable Int where
    toHaskell v =
	do  v' <- unfoldConstants v
	    case vAppView $ vDefToCon v' of
		VAVCon "zero" _ _ []	-> return 0
		VAVCon "succ" _ _ [n]	-> (1+) <$> toHaskell n
		_			-> failure $ customError $ "Not a canonical natural number: " ++ show v

instance Injectable Int where
    fromHaskell _ 0 = return v_zero
    fromHaskell t n = vApply v_succ <$> fromHaskell t (n - 1)

-- Bool -------------------------------------------------------------------

instance Extractable Bool where
    toHaskell v =
	do  v' <- unfoldConstants v
	    case vAppView $ vDefToCon v' of
		VAVCon "true" _ _ []	-> return True
		VAVCon "false" _ _ []	-> return False
		_			-> failure $ customError $ "Not a canonical boolean: " ++ show v

-- (a,b) ------------------------------------------------------------------

instance (Extractable a, Extractable b) => Extractable (a,b) where
    toHaskell v =
	do  v' <- unfoldConstants v
	    case vView v' of
		VPair v w   -> (,) <$> toHaskell v <*> toHaskell w
		_	    -> error $ "Not a canonical pair: " ++ show v

-- [a] --------------------------------------------------------------------

instance Extractable a => Extractable [a] where
    toHaskell v =
	do  v' <- unfoldConstants v
	    case vAppView $ vDefToCon v' of
		VAVCon "nil" _ _ []	-> return []
		VAVCon "::" _ _ [x,xs]	-> (:) <$> toHaskell x <*> toHaskell xs
		_			-> error $ "Not a canonical list: " ++ show v
-}

